"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@dlint/core");
const RuleUnits_1 = require("../core/RuleUnits");
const RuleTargets = new Set(Object.values(core_1.RuleTarget));
function assertArgs(target, args) {
    if (!args) {
        throw new Error(`"on" field is required for "${target}" rule`);
    }
}
function warnIfArgs(target, args) {
    if (args) {
        console.warn(`"on" field is meaningless for "${target}" rule`);
    }
}
class RuleResolver {
    constructor(selfLayer, layers) {
        this.selfLayer = selfLayer;
        this.layers = layers;
    }
    resolve(expression) {
        const { positive, target, args } = this.validate(expression);
        switch (target) {
            case core_1.RuleTarget.ALL: {
                warnIfArgs(target, args);
                return positive ? new RuleUnits_1.AllowAll() : new RuleUnits_1.DisallowAll();
            }
            case core_1.RuleTarget.ALL_LAYERS: {
                warnIfArgs(target, args);
                return positive ? new RuleUnits_1.AllowAllLayers() : new RuleUnits_1.DisallowAllLayers();
            }
            case core_1.RuleTarget.ALL_PACKAGES: {
                warnIfArgs(target, args);
                return positive ? new RuleUnits_1.AllowAllPackages() : new RuleUnits_1.DisallowAllPackages();
            }
            case core_1.RuleTarget.ALL_NODEJS: {
                warnIfArgs(target, args);
                return positive ? new RuleUnits_1.AllowAllNodejs() : new RuleUnits_1.DisallowAllNodejs();
            }
            case core_1.RuleTarget.ALL_JSON: {
                warnIfArgs(target, args);
                return positive ? new RuleUnits_1.AllowAllJson() : new RuleUnits_1.DisallowAllJson();
            }
            case core_1.RuleTarget.LAYERS: {
                assertArgs(target, args);
                const invalidLayerName = args.find((name) => !this.layers.has(name));
                if (invalidLayerName) {
                    throw new Error(`Layer name "${invalidLayerName}" is invalid`);
                }
                const layers = args
                    .map((name) => this.layers.get(name))
                    .filter((layer) => Boolean(layer));
                return positive ? new RuleUnits_1.AllowLayers(layers) : new RuleUnits_1.DisallowLayers(layers);
            }
            case core_1.RuleTarget.PACKAGES: {
                assertArgs(target, args);
                return positive ? new RuleUnits_1.AllowPackages(args) : new RuleUnits_1.DisallowPackages(args);
            }
            case core_1.RuleTarget.SELF_LAYER: {
                warnIfArgs(target, args);
                const { selfLayer } = this;
                return positive
                    ? new RuleUnits_1.AllowSelfLayer(selfLayer)
                    : new RuleUnits_1.DisallowSelfLayer(selfLayer);
            }
            default:
                throw new Error('never');
        }
    }
    validate(expression) {
        const isAllowing = core_1.RuleAction.ALLOW in expression;
        const isDisallowing = core_1.RuleAction.DISALLOW in expression;
        if (isAllowing && isDisallowing) {
            throw new Error(`Can use only one of "allow" and "disallow": ${JSON.stringify(expression)}`);
        }
        if (!isAllowing && !isDisallowing) {
            throw new Error(`Must use one of "allow" and "disallow": ${JSON.stringify(expression)}`);
        }
        const target = isAllowing
            ? expression.allow
            : expression.disallow;
        if (!RuleTargets.has(target)) {
            throw new Error(`Invalid rule target name: "${target}"\nRule target must be one of ${Object.values(core_1.RuleTarget)
                .map((target) => `"${target}"`)
                .join(', ')}`);
        }
        return {
            positive: isAllowing,
            target,
            args: expression.on,
        };
    }
}
exports.RuleResolver = RuleResolver;
